/*
 * Copyright 2010-2011 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.commons.lang;

import java.io.File;
import java.io.IOException;

import java.util.ArrayList;
import java.util.List;

import com.squeakysand.commons.io.FileHelper;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ClassLoaderHelper {

    private static final Logger LOG = LoggerFactory.getLogger(ClassLoaderHelper.class);

    public static List<String> findClassNames(File rootDir) throws IOException {
        LOG.debug("looking for classes under {}", ToStringHelper.toString(rootDir));
        List<String> result = new ArrayList<String>();
        List<File> classFiles = FileHelper.findFilesByExtension(rootDir, "class");
        result = getClassNames(classFiles, rootDir);
        LOG.debug("found classes {}", ToStringHelper.toString(result));
        return result;
    }

    public static List<Class<?>> loadAllClasses(File rootDir, ClassLoader loader) throws IOException, ClassNotFoundException {
        List<String> classNames = findClassNames(rootDir);
        return loadClasses(classNames, loader);
    }

    public static List<Class<?>> loadClasses(List<String> classes, ClassLoader loader) throws ClassNotFoundException {
        List<Class<?>> result = new ArrayList<Class<?>>();
        for (String className : classes) {
            LOG.debug("attempting to load class {}", ToStringHelper.toString(className));
            Class<?> klass = loader.loadClass(className);
            result.add(klass);
        }
        return result;
    }

    private static List<String> getClassNames(List<File> files, File baseDirectory) throws IOException {
        List<String> result = new ArrayList<String>();
        for (File file : files) {
            // trim off the base directory
            String classname = file.getPath().substring(baseDirectory.getPath().length() + 1);
            // trim off the .class extension
            classname = classname.substring(0, classname.length() - 6);
            // replace slashes with periods
            classname = classname.replace('/', '.');
            result.add(classname);
        }
        return result;
    }
}
