/*
 * Copyright 2010-2011 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.commons.lang;

import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Provides common String related functionality.
 */
public final class StringUtils {

    private static final Logger LOG = LoggerFactory.getLogger(StringUtils.class);

    /**
     * Utility class, so we enforce singleton pattern by hiding default constructor.
     */
    private StringUtils() {
        super();
    }

    /**
     * DOCUMENT ME!
     *
     * @param   letters  DOCUMENT ME!
     * @return  DOCUMENT ME!
     */
    public static SortedSet<String> generateAnagrams(String letters) {
        return generateAnagrams("", letters);
    }

    /**
     * Tests if the String argument is <code>null</code>, an empty (zero-length) String, or contains only whitespace.
     *
     * @param   s  the String to test.
     * @return  <code>true</code> if the String is <code>null</code>, an empty (zero-length) String, or contains only
     *          whitespace, <code>false</code> otherwise.
     */
    public static boolean isNullOrEmpty(String s) {
        boolean result = true;
        if ((s != null) && (s.length() > 0)) {
            // we could just use something like if (s.trim().length() == 0) to test if the string is
            // empty but that creates a whole new string object just to check the length.
            final int length = s.length();
            for (int i = 0; i < length; i++) {
                if (!Character.isWhitespace(s.charAt(i))) {
                    result = false;
                    break;
                }
            }
        }
        return result;
    }

    /**
     * <p>isNotNullOrEmpty.</p>
     *
     * @param s a {@link java.lang.String} object.
     * @return a boolean.
     */
    public static boolean isNotNullOrEmpty(String s) {
        boolean result = false;
        if ((s != null) && (s.trim().length() > 0)) {
            result = true;
        }
        return result;
    }

    /**
     * <p>isEmpty.</p>
     *
     * @param s a {@link java.lang.String} object.
     * @return a boolean.
     */
    public static boolean isEmpty(String s) {
        return s.trim().length() < 1;
    }

    /**
     * <p>crop.</p>
     *
     * @param s a {@link java.lang.String} object.
     * @param maxLength a int.
     * @return a {@link java.lang.String} object.
     */
    public static String crop(String s, int maxLength) {
        return crop(s, maxLength, null);
    }

    /**
     * <p>crop.</p>
     *
     * @param s a {@link java.lang.String} object.
     * @param suffix a {@link java.lang.String} object.
     * @param maxLength a int.
     * @return a {@link java.lang.String} object.
     */
    public static String crop(String s, int maxLength, String suffix) {
        StringBuilder sb = new StringBuilder();
        if (isNotNullOrEmpty(s)) {
            sb.append(s.trim());
            if (sb.length() > maxLength) {
                sb.append(sb.substring(0, sb.length() - suffix.length()));
                sb.append(suffix);
            }
        }
        return sb.toString();
    }

    /**
     * <p>pad.</p>
     *
     * @param s a {@link java.lang.String} object.
     * @param targetLength a int.
     * @param pad a char.
     * @return a {@link java.lang.String} object.
     */
    public static String pad(String s, int targetLength, char pad) {
        throw new RuntimeException();
    }

    private static SortedSet<String> generateAnagrams(String prefix, String letters) {
        SortedSet<String> anagrams = new TreeSet<String>();
        for (int i = 0; i < letters.length(); i++) {
            String newPrefix = prefix + letters.charAt(i);
            anagrams.add(newPrefix);
            String newLetters = "";
            for (int j = 0; j < letters.length(); j++) {
                if (j != i) {
                    newLetters += letters.charAt(j);
                }
            }
            Set<String> newAnagrams = generateAnagrams(newPrefix, newLetters);
            anagrams.addAll(newAnagrams);
        }
        return anagrams;
    }
}
