/*
 * Copyright 2010-2011 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.commons.util;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Utility class containing helpful methods related to manipulating Collections.
 */
public final class CollectionUtils {

    /**
     * Hide the constructor because this is a utility class.
     */
    private CollectionUtils() {
    }

    /**
     * Creates a new {@link java.util.List} object based on a {@link java.util.Set} object.
     *
     * @param   set  the source Collection.
     * @return  a new Collection containing all of the elements from the source collection.
     * @param <T> a T object.
     */
    public static <T extends Object> List<T> toList(Set<T> set) {
        List<T> list = new ArrayList<T>();
        list.addAll(set);
        return list;
    }

    /**
     * Creates a new {@link java.util.Set} object based on a {@link java.util.List} object.
     *
     * @param <T>
     * @param   list  the source Collection.
     * @return  a new Collection containing only the unique elements from the source Collection.
     */
    public static <T extends Object> Set<T> toSet(List<T> list) {
        Set<T> set = new HashSet<T>();
        set.addAll(list);
        return set;
    }
}
